import typing

import modern_di
from starlette.requests import Request
from starlette.types import ASGIApp, Receive, Scope, Send


class ContainerMiddleware:
    def __init__(self, app: ASGIApp) -> None:
        self.app = app

    async def __call__(
        self,
        scope: Scope,
        receive: Receive,
        send: Send,
    ) -> None:
        if scope["type"] != "http":
            return await self.app(scope, receive, send)

        request = Request(scope, receive=receive, send=send)
        context: dict[str, typing.Any] = {"request": request}

        container: modern_di.Container = request.app.state.modern_di_container
        async with container.build_child_container(context=context) as request_container:
            request.state.modern_di_container = request_container
            return await self.app(scope, receive, send)



import dataclasses
import typing

import fastapi
from modern_di import Container, providers


T_co = typing.TypeVar("T_co", covariant=True)


@dataclasses.dataclass(slots=True, frozen=True)
class Dependency(typing.Generic[T_co]):
    dependency: providers.AbstractProvider[T_co]

    async def __call__(self, request: fastapi.Request) -> T_co:
        return await self.dependency.async_resolve(request.state.modern_di_container)


def FromDI(dependency: providers.AbstractProvider[T_co], *, use_cache: bool = True) -> T_co:  # noqa: N802
    return typing.cast(T_co, fastapi.Depends(dependency=Dependency(dependency), use_cache=use_cache))


def setup_modern_di(container: Container, app: fastapi.FastAPI) -> None:
    app.state.modern_di_container = container



# Configuration file for the Sphinx documentation builder.

# -- Project information
project = "modern-di"
copyright = "2024, Modern Python"
author = "Modern Python Community"

release = ""
version = ""

# -- General configuration
extensions = [
    "sphinx_copybutton",
    "sphinx.ext.duration",
    "sphinx.ext.doctest",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
    "myst_parser",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master/", None),
}
intersphinx_disabled_domains = ["std"]

templates_path = ["_templates"]

# -- Options for HTML output
html_theme = "sphinx_rtd_theme"

# -- Options for EPUB output
epub_show_urls = "footnote"



from modern_di_fastapi.depends import FromDI, setup_modern_di
from modern_di_fastapi.middleware import ContainerMiddleware


__all__ = [
    "ContainerMiddleware",
    "FromDI",
    "setup_modern_di",
]
